/**
 * Copyright (c) 2015-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

#import "AresWebView.h"

NSString *const AresJSNavigationScheme = @"ares-webview";
NSString *const RCTInjectJS = @"ares-webview";

@implementation AresWebView
{
  UIWebView *_webView;
  NSString *_injectedJavaScript;

}

- (void)dealloc
{
  _webView.delegate = nil;
}

- (instancetype)initWithFrame:(CGRect)frame
{
  if ((self = [super initWithFrame:frame])) {
    super.backgroundColor = [UIColor clearColor];
    _automaticallyAdjustContentInsets = YES;
    _contentInset = UIEdgeInsetsZero;
    _webView = [[UIWebView alloc] initWithFrame:self.bounds];
    _webView.delegate = self;
    _webView.scalesPageToFit = YES;
    _webView.scrollView.bounces = false;
    [self addSubview:_webView];
  }
  return self;
}


- (void)setSource:(NSDictionary *)source
{
  if (![_source isEqualToDictionary:source]) {
    _source = [source copy];
    
    // Check for a static html source first
    NSString *html = [RCTConvert NSString:source[@"html"]];
    if (html) {
      NSURL *baseURL = [RCTConvert NSURL:source[@"baseUrl"]];
      if (!baseURL) {
        baseURL = [NSURL URLWithString:@"about:blank"];
      }
      [_webView loadHTMLString:html baseURL:baseURL];
      return;
    }
    
    NSURLRequest *request = [RCTConvert NSURLRequest:source];
    // Because of the way React works, as pages redirect, we actually end up
    // passing the redirect urls back here, so we ignore them if trying to load
    // the same url. We'll expose a call to 'reload' to allow a user to load
    // the existing page.
    if ([request.URL isEqual:_webView.request.URL]) {
      return;
    }
    if (!request.URL) {
      // Clear the webview
      [_webView loadHTMLString:@"" baseURL:nil];
      return;
    }
    [_webView loadRequest:request];
  }
}

- (void)layoutSubviews
{
  [super layoutSubviews];
  _webView.frame = self.bounds;
}

- (void)setContentInset:(UIEdgeInsets)contentInset
{
  _contentInset = contentInset;
  [RCTView autoAdjustInsetsForView:self
                    withScrollView:_webView.scrollView
                      updateOffset:NO];
}

- (void)setScalesPageToFit:(BOOL)scalesPageToFit
{
  if (_webView.scalesPageToFit != scalesPageToFit) {
    _webView.scalesPageToFit = scalesPageToFit;
    [_webView reload];
  }
}

- (BOOL)scalesPageToFit
{
  return _webView.scalesPageToFit;
}

- (void)setBackgroundColor:(UIColor *)backgroundColor
{
  CGFloat alpha = CGColorGetAlpha(backgroundColor.CGColor);
  self.opaque = _webView.opaque = (alpha == 1.0);
  _webView.backgroundColor = backgroundColor;
}

- (UIColor *)backgroundColor
{
  return _webView.backgroundColor;
}

- (void)refreshContentInset
{
  [RCTView autoAdjustInsetsForView:self
                    withScrollView:_webView.scrollView
                      updateOffset:YES];
}

#pragma mark - UIWebViewDelegate methods

- (BOOL)webView:(__unused UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request
 navigationType:(UIWebViewNavigationType)navigationType
{
  BOOL isJSNavigation = [request.URL.scheme isEqualToString:AresJSNavigationScheme];
  
  static NSDictionary<NSNumber *, NSString *> *navigationTypes;
  static dispatch_once_t onceToken;
  dispatch_once(&onceToken, ^{
    navigationTypes = @{
                        @(UIWebViewNavigationTypeLinkClicked): @"click",
                        @(UIWebViewNavigationTypeFormSubmitted): @"formsubmit",
                        @(UIWebViewNavigationTypeBackForward): @"backforward",
                        @(UIWebViewNavigationTypeReload): @"reload",
                        @(UIWebViewNavigationTypeFormResubmitted): @"formresubmit",
                        @(UIWebViewNavigationTypeOther): @"other",
                        };
  });
  
  // skip this for the JS Navigation handler
  if (!isJSNavigation && _onShouldStartLoadWithRequestKey) {
      _onShouldStartLoadWithRequestKey(@{@"params":@[webView.request.URL.absoluteString?:@""]});
  }
  
  // JS Navigation handler
  return !isJSNavigation;
}

- (void)webView:(__unused UIWebView *)webView didFailLoadWithError:(NSError *)error
{
//  if (_onLoadingError) {
//    if ([error.domain isEqualToString:NSURLErrorDomain] && error.code == NSURLErrorCancelled) {
//      // NSURLErrorCancelled is reported when a page has a redirect OR if you load
//      // a new URL in the WebView before the previous one came back. We can just
//      // ignore these since they aren't real errors.
//      // http://stackoverflow.com/questions/1024748/how-do-i-fix-nsurlerrordomain-error-999-in-iphone-3-0-os
//      return;
//    }
//    
//      NSMutableDictionary<NSString *, id> *event = [self baseEvent:webView];
//    [event addEntriesFromDictionary:@{
//                                      @"domain": error.domain,
//                                      @"code": @(error.code),
//                                      @"description": error.localizedDescription,
//                                      }];
//    _onLoadingError(event);
//  }
}
-(void)webViewDidStartLoad:(UIWebView*)webView{
    if (_onPageFinishedKey  && ![webView.request.URL.absoluteString isEqualToString:@"about:blank"]) {
        _onPageFinishedKey(@{@"params":@[webView.request.URL.absoluteString?:@""]});
    }
}
- (void)webViewDidFinishLoad:(UIWebView *)webView
{
    if (_injectedJavaScript != nil) {
        
    }
    // we only need the final 'finishLoad' call so only fire the event when we're actually done loading.
    else if (_onPageFinishedKey && !webView.loading && ![webView.request.URL.absoluteString isEqualToString:@"about:blank"]) {
        _onPageFinishedKey(@{@"params":@[webView.request.URL.absoluteString?:@""]});
    }
}
-(void)loadJs:(NSString *)jsString{
  [_webView stringByEvaluatingJavaScriptFromString:jsString];
}
-(void)loadUrl:(NSString *)url{
  NSURL * _url = [NSURL URLWithString:url];
  NSURLRequest *request = [NSURLRequest requestWithURL:_url];
  // the existing page
  if ([request.URL isEqual:_webView.request.URL]) {
    return;
  }
  if (!request.URL) {
    // Clear the webview
    [_webView loadHTMLString:@"" baseURL:nil];
    return;
  }
  [_webView loadRequest:request];

}
- (void)goForward
{
    [_webView goForward];
}

- (void)goBack
{
    [_webView goBack];
}

- (void)reload
{
    NSURLRequest *request = [RCTConvert NSURLRequest:self.source];
    if (request.URL && !_webView.request.URL.absoluteString.length) {
        [_webView loadRequest:request];
    }
    else {
        [_webView reload];
    }
}


@end
