/**
 * @descriptin - 拍照
 */
'use strict'
import React, { Component } from 'react';
import {
    NativeAppEventEmitter,
    NativeModules,
    Platform,
    StyleSheet,
    requireNativeComponent,
    View,
    ViewPropTypes
} from 'react-native';
import PropTypes from 'prop-types';

const CameraManager = NativeModules.CameraManager||NativeModules.Camera;
const CAMERA_REF = 'camera';

function convertNativeProps(props) {
    const newProps = { ...props };
    if (typeof props.aspect === 'string') {
        newProps.aspect = Camera.Aspect[props.aspect];
    }

    if (typeof props.flashMode === 'string') {
        newProps.flashMode = Camera.FlashMode[props.flashMode];
    }

    if (typeof props.orientation === 'string') {
        newProps.orientation = Camera.Orientation[props.orientation];
    }

    if (typeof props.torchMode === 'string') {
        newProps.torchMode = Camera.TorchMode[props.torchMode];
    }

    if (typeof props.type === 'string') {
        newProps.type = Camera.Type[props.type];
    }

    if (typeof props.captureQuality === 'string') {
        newProps.captureQuality = Camera.CaptureQuality[props.captureQuality];
    }

    if (typeof props.captureMode === 'string') {
        newProps.captureMode = Camera.CaptureMode[props.captureMode];
    }

    // do not register barCodeTypes if no barcode listener
    if (typeof props.onBarCodeRead !== 'function') {
        newProps.barCodeTypes = [];
    }

    return newProps;
}

module.exports = class Camera extends Component {



    constructor() {
        super();
        this.state = {
            isAuthorized: false,
            isRecording: false
        };
    }

    async componentWillMount() {
        this.cameraBarCodeReadListener = NativeAppEventEmitter.addListener('CameraBarCodeRead', this._onBarCodeRead);

        let { captureMode } = convertNativeProps({ captureMode: this.props.captureMode })
        let hasVideoAndAudio = this.props.captureAudio && captureMode === Camera.CaptureMode.video
        let check = hasVideoAndAudio ? Camera.checkDeviceAuthorizationStatus : Camera.checkVideoAuthorizationStatus;

        if (check) {
            const isAuthorized = await check();
            this.setState({ isAuthorized });
        }
    }

    componentWillUnmount() {
        this.cameraBarCodeReadListener.remove();

        if (this.state.isRecording) {
            this.stopCapture();
        }
    }

    render() {
        const nativeProps = convertNativeProps(this.props);

        return (
            <RCTCamera
                ref={CAMERA_REF}
                {...nativeProps}
                style = {this.props.style}/>
        );
    }

    _onBarCodeRead(data) {
        this.props.onBarCodeRead&&this.props.onBarCodeRead(data);
    };
    setNativeProps(props) {
        this.refs[CAMERA_REF].setNativeProps(props);
    }
    capture(options) {
        const props = convertNativeProps(this.props);
        options = {
            audio: props.captureAudio,
            barCodeTypes: props.barCodeTypes,
            mode: props.captureMode,
            playSoundOnCapture: props.playSoundOnCapture,
            target: props.captureTarget,
            quality: props.captureQuality,
            type: props.type,
            title: '',
            description: '',
             ...options
        };

        if (options.mode === Camera.CaptureMode.video) {
            options.totalSeconds = (options.totalSeconds > -1 ? options.totalSeconds : -1);
            options.preferredTimeScale = options.preferredTimeScale || 30;
            this.setState({ isRecording: true });
        }

        return CameraManager.capture(options);
    }

    stopCapture() {
        if (this.state.isRecording) {
            CameraManager.stopCapture();
            this.setState({ isRecording: false });
        }
    }

    getFOV() {
        return CameraManager.getFOV();
    }

    hasFlash() {
        if (Platform.OS === 'android') {
            const props = convertNativeProps(this.props);
            return CameraManager.hasFlash({
                type: props.type
            });
        }
        return CameraManager.hasFlash();
    }
}
Camera.Aspect = CameraManager.Aspect;
Camera.BarCodeType = CameraManager.BarCodeType;
Camera.Type = CameraManager.Type;
Camera.CaptureMode = CameraManager.CaptureMode;
Camera.CaptureTarget = CameraManager.CaptureTarget;
Camera.CaptureQuality = CameraManager.CaptureQuality;
Camera.Orientation = CameraManager.Orientation;
Camera.FlashMode = CameraManager.FlashMode;
Camera.TorchMode = CameraManager.TorchMode;
Camera.checkDeviceAuthorizationStatus = CameraManager.checkDeviceAuthorizationStatus;
Camera.checkVideoAuthorizationStatus = CameraManager.checkVideoAuthorizationStatus;
Camera.checkAudioAuthorizationStatus = CameraManager.checkAudioAuthorizationStatus;

Camera.propTypes = {
    ...ViewPropTypes,
    aspect: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ]),
    captureAudio: PropTypes.bool,
    captureMode: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ]),
    captureQuality: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ]),
    captureTarget: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ]),
    defaultOnFocusComponent: PropTypes.bool,
    flashMode: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ]),
    keepAwake: PropTypes.bool,
    onBarCodeRead: PropTypes.func,
    onFocusChanged: PropTypes.func,
    onZoomChanged: PropTypes.func,
    mirrorImage: PropTypes.bool,
    barCodeTypes: PropTypes.array,
    orientation: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ]),
    playSoundOnCapture: PropTypes.bool,
    torchMode: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ]),
    type: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ])
};
Camera.defaultProps = {
    aspect: CameraManager.Aspect.fill,
    type: CameraManager.Type.back,
    orientation: CameraManager.Orientation.auto,
    captureAudio: true,
    captureMode: CameraManager.CaptureMode.still,
    captureTarget: CameraManager.CaptureTarget.cameraRoll,
    captureQuality: CameraManager.CaptureQuality.high,
    defaultOnFocusComponent: true,
    flashMode: CameraManager.FlashMode.off,
    playSoundOnCapture: true,
    torchMode: CameraManager.TorchMode.off,
    mirrorImage: false,
    barCodeTypes: Object.values(CameraManager.BarCodeType),
};
const RCTCamera = requireNativeComponent('RCTCamera', Camera);
